# img_px_size in mm
gsecars_photo = {'img_px_size_hor': 0.00035,
                 'img_px_size_ver': 0.00035,
                 'img_hor_px': 1920,
                 'img_ver_px': 1200,
                 'flip_hor_prefixes': 'us,ms',
                 'flip_hor': False,
                 'flip_ver': True,
                 }
