/*  ------------------------------------------------------------------------  */
/* QtMaterial - https://github.com/UN-GCPDS/qt-material
/* By Yeison Cardona - GCPDS
/*  ------------------------------------------------------------------------  */

*{
  color: #f1f1f1;

  font-family: Roboto;

  
    font-size: 11px;
  

  
    line-height: 4px;
  

  selection-background-color: #0084ff;
  selection-color: #f1f1f1;
}

*:focus {
   outline: none;
}

/*  ------------------------------------------------------------------------  */
/*  Custom colors  */

.danger{
  color: #dc3545;
  background-color: transparent;
}

.warning{
  color: #ffc107;
  background-color: transparent;
}

.success{
  color: #17a2b8;
  background-color: transparent;
}

.danger:disabled{
  color: rgba(220, 53, 69, 0.4);
  border-color: rgba(220, 53, 69, 0.4);
}

.warning:disabled{
  color: rgba(255, 193, 7, 0.4);
  border-color: rgba(255, 193, 7, 0.4);
}

.success:disabled{
  color: rgba(23, 162, 184, 0.4);
  border-color: rgba(23, 162, 184, 0.4);
}

.danger:flat:disabled{
  background-color: rgba(220, 53, 69, 0.1);
}

.warning:flat:disabled{
  background-color: rgba(255, 193, 7, 0.1);
}

.success:flat:disabled{
  background-color: rgba(23, 162, 184, 0.1);
}

/*  ------------------------------------------------------------------------  */
/*  Basic widgets  */

QWidget {
  background-color: #31363b;
}

QGroupBox,
QFrame {
  background-color: #31363b;
  border: 2px solid #AF770F;
  border-radius: 4px;
}

QGroupBox.fill_background,
QFrame.fill_background {
  background-color: #232629;
  border: 2px solid #232629;
  border-radius: 4px;
}

QSplitter {
  background-color: transparent;
  border: none
}

QStatusBar {
  color: #f1f1f1;
  background-color: rgba(175, 119, 15, 0.2);
  border-radius: 0px;
}

QScrollArea,
QStackedWidget,
QWidget > QToolBox,
QToolBox > QWidget,
QTabWidget > QWidget {
  border: none;
}

QTabWidget::pane {
  border: none;
}

/*  ------------------------------------------------------------------------  */
/*  Inputs  */

QDateEdit,
QDateTimeEdit,
QSpinBox,
QDoubleSpinBox,
QTextEdit,
QLineEdit,
QPushButton {
  color: #ff9500;
  background-color: #31363b;
  border: 2px solid #ff9500;
  border-radius: 4px;
  height: 24px;
}

QDateEdit,
QDateTimeEdit,
QSpinBox,
QDoubleSpinBox,
QTreeView,
QListView,
QLineEdit,
QComboBox {
  color: #f1f1f1;
  padding-left: 8px;
  border-radius: 0px;
  border-radius: 0px;
  border-top-left-radius: 4px;
  border-top-right-radius: 4px;
  height: 24px;
  background-color: rgba(35, 38, 41, 0.75);
  border: 2px solid rgba(241, 241, 241, 0.2);
  border-width: 0 0 2px 0;
}

QPlainTextEdit {
  border-radius: 4px;
  padding: 4px 8px;
  background-color: #31363b;
  border: 2px solid #AF770F;
}

QTextEdit {
  padding: 4px 8px;
  border-radius: 4px;
  background-color: #232629;
}

QDateEdit:disabled,
QDateTimeEdit:disabled,
QSpinBox:disabled,
QDoubleSpinBox:disabled,
QTextEdit:disabled,
QLineEdit:disabled {
  color: rgba(241, 241, 241, 0.2);
  background-color: rgba(35, 38, 41, 0.3);
  border: 2px solid #232629;
  border-width: 0 0 2px 0;
  padding: 0px 8px;
  border-radius: 0px;
  border-top-left-radius: 4px;
  border-top-right-radius: 4px;
  height: 24px;
}

/*  ------------------------------------------------------------------------  */
/*  QComboBox  */

QDateEdit,
QComboBox {
  color: #f1f1f1;
  border: 2px solid #ff9500;
  border-radius: 0px;
  border-top-left-radius: 4px;
  border-top-right-radius: 4px;
  height: 24px;
  background-color: rgba(35, 38, 41, 0.75);
  border: 2px solid rgba(241, 241, 241, 0.2);
  border-width: 0 0 2px 0;
}

QDateEdit:disabled,
QComboBox:disabled {
  color: rgba(241, 241, 241, 0.2);
  background-color: rgba(35, 38, 41, 0.3);
  border-bottom: 2px solid #232629;
}

QDateEdit::drop-down,
QComboBox::drop-down {
  border: none;
  color: #ff9500;
  width: 20px;
}

QDateEdit::down-arrow,
QComboBox::down-arrow {
  image: url(icon:/active/downarrow.svg);
  margin-right: 4px;
}


QDateEdit::down-arrow:focus,
QComboBox::down-arrow:focus {
  image: url(icon:/primary/downarrow.svg);
  margin-right: 4px;
}

QDateEdit::down-arrow:disabled,
QComboBox::down-arrow:disabled {
  image: url(icon:/disabled/downarrow.svg);
  margin-right: 4px;
}

QDateEdit QAbstractItemView,
QComboBox QAbstractItemView {
  background-color: #232629;
  border: 2px solid #AF770F;
  border-radius: 4px;
}

QDateEdit[frame='false'],
QComboBox[frame='false'] {
  color: #f1f1f1;
  background-color: transparent;
  border: 1px solid transparent;
}

QDateEdit[frame='false']:disabled,
QComboBox[frame='false']:disabled {
  color: rgba(241, 241, 241, 0.2);
}

/*  ------------------------------------------------------------------------  */
/*  Spin buttons  */

QDateTimeEdit::up-button,
QDoubleSpinBox::up-button,
QSpinBox::up-button {
  subcontrol-origin: border;
  subcontrol-position: top right;
  width: 20px;
  image: url(icon:/active/uparrow.svg);
  border-width: 0px;
  margin-right: 5px;
}

QDateTimeEdit::up-button:disabled,
QDoubleSpinBox::up-button:disabled,
QSpinBox::up-button:disabled {
  image: url(icon:/disabled/uparrow.svg);
}

QDateTimeEdit::down-button,
QDoubleSpinBox::down-button,
QSpinBox::down-button {
  subcontrol-origin: border;
  subcontrol-position: bottom right;
  width: 20px;
  image: url(icon:/active/downarrow.svg);
  border-width: 0px;
  border-top-width: 0;
  margin-right: 5px;
}

QDateTimeEdit::down-button:disabled,
QDoubleSpinBox::down-button:disabled,
QSpinBox::down-button:disabled {
  image: url(icon:/disabled/downarrow.svg);
}

/*  ------------------------------------------------------------------------  */
/*  QPushButton  */

QPushButton {
  text-transform: uppercase;
  margin: 0px;
  padding: 4px 8px;
  height: 24px;
  font-weight: bold;

  
    border-radius: 4px;
  
}

QPushButton:checked,
QPushButton:pressed {
  color: #31363b;
  background-color: #ff9500;
}

QPushButton:flat {
  margin: 0px;
  color: #ff9500;
  border: none;
  background-color: transparent;
}

QPushButton:flat:hover {
  background-color: rgba(255, 149, 0, 0.1);
}

QPushButton:flat:pressed,
QPushButton:flat:checked {
  background-color: rgba(255, 149, 0, 0.1);
}

QPushButton:disabled {
  color: rgba(175, 119, 15, 0.75);
  background-color: transparent;
  border-color:  #AF770F;
}

QPushButton:flat:disabled {
  color: rgba(175, 119, 15, 0.75);
  background-color: rgba(175, 119, 15, 0.25);
  border: none;
}

QPushButton:disabled {
  border: 2px solid rgba(175, 119, 15, 0.75);
}

QPushButton:checked:disabled {
  color: #232629;
  background-color: #AF770F;
  border-color:  #AF770F;
}

QToolButton:focus,
QPushButton:focus {
  background-color: rgba(255, 149, 0, 0.2);
}

QPushButton:checked:focus,
QPushButton:pressed:focus {
  background-color: rgba(255, 149, 0, 0.8);
}

QPushButton:flat:focus {
  border: none;
  background-color: rgba(255, 149, 0, 0.2);
}


/*  ------------------------------------------------------------------------  */
/*  QTabBar  */

QTabBar{
  text-transform: uppercase;
  font-weight: bold;
}

QTabBar::tab {
  color: #f1f1f1;
  border: 0px;
}

QTabBar::tab:bottom,
QTabBar::tab:top{
  padding: 0 8px;
  height: 20px;
}

QTabBar::tab:left,
QTabBar::tab:right{
  padding: 8px 0;
  width: 20px;
}

QTabBar::tab:top:selected,
QTabBar::tab:top:hover {
  color: #ff9500;
  border-bottom: 2px solid #ff9500;
}

QTabBar::tab:bottom:selected,
QTabBar::tab:bottom:hover {
  color: #ff9500;
  border-top: 2px solid #ff9500;
}

QTabBar::tab:right:selected,
QTabBar::tab:right:hover {
  color: #ff9500;
  border-left: 2px solid #ff9500;
}

QTabBar::tab:left:selected,
QTabBar::tab:left:hover {
  color: #ff9500;
  border-right: 2px solid #ff9500;
}

QTabBar QToolButton:hover,
QTabBar QToolButton {
  border: 0px;
  background-color: #232629;
  background: #232629;
}

QTabBar QToolButton::up-arrow {
  image: url(icon:/primary/uparrow2.svg);
  width: 20px;
}

QTabBar QToolButton::down-arrow {
  image: url(icon:/primary/downarrow2.svg);
  width: 20px;
}

QTabBar QToolButton::right-arrow {
  image: url(icon:/disabled/rightarrow2.svg);
  height: 20px;
}

QTabBar QToolButton::left-arrow {
  image: url(icon:/disabled/leftarrow2.svg);
  height: 20px;
}

QTabBar::close-button {
  image: url(icon:/primary/tab_close.svg);
}

QTabBar::close-button:hover {
  image: url(icon:/primary/tab_close.svg);
}

/*  ------------------------------------------------------------------------  */
/*  QGroupBox  */

QGroupBox {
  padding: 8px;
  padding-top: 28px;
  line-height: ;
  text-transform: uppercase;
  font-size: ;
}

QGroupBox::title {
  color: rgba(241, 241, 241, 0.4);
  subcontrol-origin: margin;
  subcontrol-position: top left;
  padding: 8px;
  background-color: #31363b;
  background-color: transparent;
  height: 28px;
}

/*  ------------------------------------------------------------------------  */
/*  QRadioButton and QCheckBox labels  */

QRadioButton,
QCheckBox {
  spacing: 4px;
  color: #f1f1f1;
  line-height: 14px;
  height: 28px;
  background-color: transparent;
  spacing: 5px;
}

QRadioButton:disabled,
QCheckBox:disabled {
  color: rgba(241, 241, 241, 0.3);
}

/*  ------------------------------------------------------------------------  */
/*  General Indicators  */

QGroupBox::indicator {
  width: 16px;
  height: 16px;
  border-radius: 3px;
}

QMenu::indicator,
QListView::indicator,
QTableWidget::indicator,
QRadioButton::indicator,
QCheckBox::indicator {
  width: 20px;
  height: 20px;
  border-radius: 4px;
 }

/*  ------------------------------------------------------------------------  */
/*  QListView Indicator  */

QListView::indicator:checked,
QListView::indicator:checked:selected,
QListView::indicator:checked:focus {
  image: url(icon:/primary/checklist.svg);
}

QListView::indicator:checked:selected:active {
  image: url(icon:/primary/checklist_invert.svg);
}

QListView::indicator:checked:disabled {
  image: url(icon:/disabled/checklist.svg);
}

QListView::indicator:indeterminate,
QListView::indicator:indeterminate:selected,
QListView::indicator:indeterminate:focus {
  image: url(icon:/primary/checklist_indeterminate.svg);
}

QListView::indicator:indeterminate:selected:active {
  image: url(icon:/primary/checklist_indeterminate_invert.svg);
}

QListView::indicator:indeterminate:disabled {
  image: url(icon:/disabled/checklist_indeterminate.svg);
}

/*  ------------------------------------------------------------------------  */
/*  QTableView Indicator  */

QTableView::indicator:enabled:checked,
QTableView::indicator:enabled:checked:selected,
QTableView::indicator:enabled:checked:focus {
  image: url(icon:/primary/checkbox_checked.svg);
}

QTableView::indicator:checked:selected:active {
  image: url(icon:/primary/checkbox_checked_invert.svg);
}

QTableView::indicator:disabled:checked,
QTableView::indicator:disabled:checked:selected,
QTableView::indicator:disabled:checked:focus {
  image: url(icon:/disabled/checkbox_checked.svg);
}

QTableView::indicator:enabled:unchecked,
QTableView::indicator:enabled:unchecked:selected,
QTableView::indicator:enabled:unchecked:focus {
  image: url(icon:/primary/checkbox_unchecked.svg);
}

QTableView::indicator:unchecked:selected:active {
  image: url(icon:/primary/checkbox_unchecked_invert.svg);
}

QTableView::indicator:disabled:unchecked,
QTableView::indicator:disabled:unchecked:selected,
QTableView::indicator:disabled:unchecked:focus {
  image: url(icon:/disabled/checkbox_unchecked.svg);
}

QTableView::indicator:enabled:indeterminate,
QTableView::indicator:enabled:indeterminate:selected,
QTableView::indicator:enabled:indeterminate:focus {
  image: url(icon:/primary/checkbox_indeterminate.svg);
}

QTableView::indicator:indeterminate:selected:active {
  image: url(icon:/primary/checkbox_indeterminate_invert.svg);
}

QTableView::indicator:disabled:indeterminate,
QTableView::indicator:disabled:indeterminate:selected,
QTableView::indicator:disabled:indeterminate:focus {
  image: url(icon:/disabled/checkbox_indeterminate.svg);
}

/*  ------------------------------------------------------------------------  */
/*  QCheckBox and QGroupBox Indicator  */

QCheckBox::indicator:checked,
QGroupBox::indicator:checked {
  image: url(icon:/primary/checkbox_checked.svg);
}

QCheckBox::indicator:unchecked,
QGroupBox::indicator:unchecked {
  image: url(icon:/primary/checkbox_unchecked.svg);
}

QCheckBox::indicator:indeterminate,
QGroupBox::indicator:indeterminate {
  image: url(icon:/primary/checkbox_indeterminate.svg);
}

QCheckBox::indicator:checked:disabled,
QGroupBox::indicator:checked:disabled {
  image: url(icon:/disabled/checkbox_checked.svg);
}

QCheckBox::indicator:unchecked:disabled,
QGroupBox::indicator:unchecked:disabled {
  image: url(icon:/disabled/checkbox_unchecked.svg);
}

QCheckBox::indicator:indeterminate:disabled,
QGroupBox::indicator:indeterminate:disabled {
  image: url(icon:/disabled/checkbox_indeterminate.svg);
}

/*  ------------------------------------------------------------------------  */
/*  QRadioButton Indicator  */

QRadioButton::indicator:checked {
  image: url(icon:/primary/radiobutton_checked.svg);
}

QRadioButton::indicator:unchecked {
  image: url(icon:/primary/radiobutton_unchecked.svg);
}

QRadioButton::indicator:checked:disabled {
  image: url(icon:/disabled/radiobutton_checked.svg);
}

QRadioButton::indicator:unchecked:disabled {
  image: url(icon:/disabled/radiobutton_unchecked.svg);
}


/*  ------------------------------------------------------------------------  */
/*  QDockWidget  */

QDockWidget {
  color: #f1f1f1;
  text-transform: uppercase;
  border: 2px solid #232629;
  titlebar-close-icon: url(icon:/primary/close.svg);
  titlebar-normal-icon: url(icon:/primary/float.svg);
  border-radius: 4px;
}

QDockWidget::title {
  text-align: left;
  padding-left: 28px;
  padding: 3px;
  margin-top: 4px;
}

/*  ------------------------------------------------------------------------  */
/*  QComboBox indicator  */

QComboBox::indicator:checked {
  image: url(icon:/primary/checklist.svg);
}

QComboBox::indicator:checked:selected {
  image: url(icon:/primary/checklist_invert.svg);
}

/*  ------------------------------------------------------------------------  */
/*  Menu Items  */

QComboBox::item,
QCalendarWidget QMenu::item,
QMenu::item {
  
    height: 20px;
  
  border: 8px solid transparent;
  color: #f1f1f1;
}

QCalendarWidget QMenu::item,
QMenu::item {
  
    
      padding: 0px 16px 0px 8px;  /* pyside6 or pyqt6 */
    
  
}


QComboBox::item:selected,
QCalendarWidget QMenu::item:selected,
QMenu::item:selected {
  color: #f1f1f1;
  background-color: #0084ff;
  border-radius: 0px;
}

QComboBox::item:disabled,
QCalendarWidget QMenu::item:disabled,
QMenu::item:disabled {
  color: rgba(241, 241, 241, 0.3);
}

/*  ------------------------------------------------------------------------  */
/*  QMenu  */

QCalendarWidget QMenu,
QMenu {
  background-color: #232629;
  border: 2px solid #AF770F;
  border-radius: 4px;
}

QMenu::separator {
  height: 2px;
  background-color: #AF770F;
  margin-left: 2px;
  margin-right: 2px;
}

QMenu::right-arrow{
  image: url(icon:/primary/rightarrow.svg);
  width: 8px;
  height: 8px;
}

QMenu::right-arrow:selected{
  image: url(icon:/disabled/rightarrow.svg);
}

QMenu::indicator:non-exclusive:unchecked {
  image: url(icon:/primary/checkbox_unchecked.svg);
}

QMenu::indicator:non-exclusive:unchecked:selected {
  image: url(icon:/primary/checkbox_unchecked_invert.svg);
}

QMenu::indicator:non-exclusive:checked {
  image: url(icon:/primary/checkbox_checked.svg);
}

QMenu::indicator:non-exclusive:checked:selected {
  image: url(icon:/primary/checkbox_checked_invert.svg);
}

QMenu::indicator:exclusive:unchecked {
  image: url(icon:/primary/radiobutton_unchecked.svg);
}

QMenu::indicator:exclusive:unchecked:selected {
  image: url(icon:/primary/radiobutton_unchecked_invert.svg);
}

QMenu::indicator:exclusive:checked {
  image: url(icon:/primary/radiobutton_checked.svg);
}

QMenu::indicator:exclusive:checked:selected {
  image: url(icon:/primary/radiobutton_checked_invert.svg);
}

/*  ------------------------------------------------------------------------  */
/*  QMenuBar  */

QMenuBar {
  background-color: #232629;
  color: #f1f1f1;
}

QMenuBar::item {
  height: 24px;
  padding: 8px;
  background-color: transparent;
  color: #f1f1f1;
}

QMenuBar::item:selected,
QMenuBar::item:pressed {
  color: #f1f1f1;
  background-color: #0084ff;
}

/*  ------------------------------------------------------------------------  */
/*  QToolBox  */

QToolBox::tab {
  background-color: #232629;
  color: #f1f1f1;
  text-transform: uppercase;
  border-radius: 4px;
  padding-left: 15px;
}

QToolBox::tab:selected,
QToolBox::tab:hover {
  background-color: rgba(255, 149, 0, 0.2);
}

/*  ------------------------------------------------------------------------  */
/*  QProgressBar  */

QProgressBar {
  border-radius: 0;
  background-color: #AF770F;
  text-align: center;
  color: transparent;
}

QProgressBar::chunk {
  background-color: #ff9500;
}

/*  ------------------------------------------------------------------------  */
/*  QScrollBar  */

QScrollBar:horizontal {
  border: 0;
  background: #232629;
  height: 4px;
}

QScrollBar:vertical {
  border: 0;
  background: #232629;
  width: 4px;
}

QScrollBar::handle {
  background: rgba(255, 149, 0, 0.1);
}

QScrollBar::handle:horizontal {
  min-width: 16px;
}

QScrollBar::handle:vertical {
  min-height: 16px;
}

QScrollBar::handle:vertical:hover,
QScrollBar::handle:horizontal:hover {
  background: #ff9500;
}

QScrollBar::add-line:vertical,
QScrollBar::sub-line:vertical,
QScrollBar::add-line:horizontal,
QScrollBar::sub-line:horizontal {
  border: 0;
  background: transparent;
  width: 0px;
  height: 0px;
}

QScrollBar::sub-page:horizontal,
QScrollBar::add-page:horizontal,
QScrollBar::sub-page:vertical,
QScrollBar::add-page:vertical,
QScrolLBar:vertical {
    background: transparent;
}

/*  ------------------------------------------------------------------------  */
/*  QScrollBar-Big  */

QScrollBar.big:horizontal {
  border: 0;
  background: #232629;
  height: 28px;
}

QScrollBar.big:vertical {
  border: 0;
  background: #232629;
  width: 28px;
}

QScrollBar.big::handle,
QScrollBar.big::handle:vertical:hover,
QScrollBar.big::handle:horizontal:hover {
  background: #ff9500;
}

QScrollBar.big::handle:horizontal {
  min-width: 16px;
}

QScrollBar.big::handle:vertical {
  min-height: 16px;
}

QScrollBar.big::add-line:vertical,
QScrollBar.big::sub-line:vertical,
QScrollBar.big::add-line:horizontal,
QScrollBar.big::sub-line:horizontal {
  border: 0;
  background: transparent;
  width: 0px;
  height: 0px;
}

/*  ------------------------------------------------------------------------  */
/*  QSlider  */

QSlider:horizontal {
  min-height: 16px;
  max-height: 16px;
}

QSlider:vertical {
  min-width: 16px;
  max-width: 16px;
}

QSlider::groove:horizontal {
  height: 4px;
  background: #393939;
  margin: 0 4px;
}

QSlider::groove:vertical {
  width: 4px;
  background: #393939;
  margin: 4px 0;
}

QSlider::handle:horizontal {
  image: url(icon:/primary/slider.svg);
  width: 10px;
  height: 10px;
  margin: -10px -1px;
}

QSlider::handle:vertical {
  image: url(icon:/primary/slider.svg);
  width: 10px;
  height: 10px;
  margin: -1px -10px;
}

QSlider::add-page {
background: #232629;
}

QSlider::sub-page {
background: #ff9500;
}

/*  ------------------------------------------------------------------------  */
/*  QLabel  */

QLabel {
  border: none;
  background: transparent;
  color: #f1f1f1
}

QLabel:disabled {
  color: rgba(241, 241, 241, 0.2)
}

/*  ------------------------------------------------------------------------  */
/*  VLines and HLinex  */

QFrame[frameShape="4"] {
    border-width: 1px 0 0 0;
    background: none;
}

QFrame[frameShape="5"] {
    border-width: 0 1px 0 0;
    background: none;
}

QFrame[frameShape="4"],
QFrame[frameShape="5"] {
  border-color: #AF770F;
}

/*  ------------------------------------------------------------------------  */
/*  QToolBar  */

QToolBar {
  background: #31363b;
  border: 0px solid;
}

QToolBar:horizontal {
  border-bottom: 1px solid #AF770F;
}

QToolBar:vertical {
  border-right: 1px solid #AF770F;
}

QToolBar::handle:horizontal {
  image: url(icon:/primary/toolbar-handle-horizontal.svg);
}

QToolBar::handle:vertical {
  image: url(icon:/primary/toolbar-handle-vertical.svg);
}

QToolBar::separator:horizontal {
  border-right: 1px solid #AF770F;
  border-left: 1px solid #AF770F;
  width: 1px;
}

QToolBar::separator:vertical {
  border-top: 1px solid #AF770F;
  border-bottom: 1px solid #AF770F;
  height: 1px;
}


/*  ------------------------------------------------------------------------  */
/*  QToolButton  */

QToolButton {
  background: #31363b;
  border: 0px;
  height: 28px;
  margin: 3px;
  padding: 3px;
  border-right: 12px solid #31363b;
  border-left: 12px solid #31363b;
}

QToolButton:hover {
  background: #AF770F;
  border-right: 12px solid #AF770F;
  border-left: 12px solid #AF770F;
}

QToolButton:pressed {
  background: #232629;
  border-right: 12px solid #232629;
  border-left: 12px solid #232629;
}

QToolButton:checked {
  background: #AF770F;
  border-left: 12px solid #AF770F;
  border-right: 12px solid #ff9500;
}

/*  ------------------------------------------------------------------------  */
/*  General viewers  */

QTableView {
  background-color: #31363b;
  border: 1px solid #232629;
  border-radius: 4px;
}

QTreeView,
QListView {
  border-radius: 4px;
  padding: 4px;
  margin: 0px;
  border: 0px;
}

QTableView::item,
QTreeView::item,
QListView::item {
  padding: 4px;
  min-height: 24px;
  color: #f1f1f1;
  selection-color: #f1f1f1; /* For Windows */
  border-color: transparent;  /* Fix #34 */
}

/*  ------------------------------------------------------------------------  */
/*  Items Selection */

QTableView::item:selected,
QTreeView::item:selected,
QListView::item:selected {
  background-color: rgba(255, 149, 0, 0.2);
  selection-background-color: rgba(255, 149, 0, 0.2);
  color: #f1f1f1;
  selection-color: #f1f1f1; /* For Windows */
}

QTableView::item:selected:focus,
QTreeView::item:selected:focus,
QListView::item:selected:focus {
  background-color: #ff9500;
  selection-background-color: #ff9500;
  color: #f1f1f1;
  selection-color: #f1f1f1; /* For Windows */
}

QTableView {
  selection-background-color: rgba(255, 149, 0, 0.2);
}

QTableView:focus {
  selection-background-color: #ff9500;
}

QTableView::item:disabled {
  color: rgba(241, 241, 241, 0.3);
  selection-color: rgba(241, 241, 241, 0.3);
  background-color: #232629;
  selection-background-color: #232629;
}

/*  ------------------------------------------------------------------------  */
/*  QTreeView  */

QTreeView::branch{
  background-color: #232629;
}

QTreeView::branch:closed:has-children:has-siblings,
QTreeView::branch:closed:has-children:!has-siblings {
  image: url(icon:/primary/branch-closed.svg);
}

QTreeView::branch:open:has-children:!has-siblings,
QTreeView::branch:open:has-children:has-siblings {
  image: url(icon:/primary/branch-open.svg);
}

QTreeView::branch:has-siblings:!adjoins-item {
  border-image: url(icon:/disabled/vline.svg) 0;
}

QTreeView::branch:has-siblings:adjoins-item {
    border-image: url(icon:/disabled/branch-more.svg) 0;
}

QTreeView::branch:!has-children:!has-siblings:adjoins-item,
QTreeView::branch:has-children:!has-siblings:adjoins-item {
    border-image: url(icon:/disabled/branch-end.svg) 0;
}

QTreeView QHeaderView::section {
  border: none;
}


/*  ------------------------------------------------------------------------  */
/*  Custom buttons  */

QPushButton.danger {
  border-color: #dc3545;
  color: #dc3545;
}

QPushButton.danger:checked,
QPushButton.danger:pressed {
  color: #31363b;
  background-color: #dc3545;
}

QPushButton.warning{
  border-color: #ffc107;
  color: #ffc107;
}

QPushButton.warning:checked,
QPushButton.warning:pressed {
  color: #31363b;
  background-color: #ffc107;
}

QPushButton.success {
  border-color: #17a2b8;
  color: #17a2b8;
}

QPushButton.success:checked,
QPushButton.success:pressed {
  color: #31363b;
  background-color: #17a2b8;
}

QPushButton.danger:flat:hover {
  background-color: rgba(220, 53, 69, 0.2);
}

QPushButton.danger:flat:pressed,
QPushButton.danger:flat:checked {
  background-color: rgba(220, 53, 69, 0.1);
  color: #dc3545;
}

QPushButton.warning:flat:hover {
  background-color: rgba(255, 193, 7, 0.2);
}

QPushButton.warning:flat:pressed,
QPushButton.warning:flat:checked {
  background-color: rgba(255, 193, 7, 0.1);
  color: #ffc107;
}

QPushButton.success:flat:hover {
  background-color: rgba(23, 162, 184, 0.2);
}

QPushButton.success:flat:pressed,
QPushButton.success:flat:checked {
  background-color: rgba(23, 162, 184, 0.1);
  color: #17a2b8;
}

/*  ------------------------------------------------------------------------  */
/*  QTableView  */

QTableCornerButton::section {
  background-color: #232629;
  border-radius: 0px;
  border-right: 1px solid;
  border-bottom: 1px solid;
  border-color: #31363b;
}

QTableView {
  alternate-background-color: rgba(35, 38, 41, 0.7);
}

QHeaderView {
  border: none;
}

QHeaderView::section {
  color: rgba(241, 241, 241, 0.7);
  text-transform: uppercase;
  background-color: #232629;
  padding: 0 16px;
  height: 28px;
  border-radius: 0px;
  border-right: 1px solid;
  border-bottom: 1px solid;
  border-color: #31363b;
}

QHeaderView::section:vertical {

}

QHeaderView::section:horizontal {

}

/*  ------------------------------------------------------------------------  */
/*  QLCDNumber  */

QLCDNumber {
  color: #ff9500;
  background-color:rgba(255, 149, 0, 0.1);
  border: 1px solid rgba(255, 149, 0, 0.3);
  border-radius: 4px;
}

/*  ------------------------------------------------------------------------  */
/*  QCalendarWidget  */

QCalendarWidget {
  min-height: 300px;
}

#qt_calendar_prevmonth {
  qproperty-icon: url(icon:/primary/leftarrow.svg);
}

#qt_calendar_nextmonth {
  qproperty-icon: url(icon:/primary/rightarrow.svg);
}

/*  ------------------------------------------------------------------------  */
/*  Inline QLineEdit  */

QTreeView QLineEdit,
QTableView QLineEdit,
QListView QLineEdit {
  color: #f1f1f1;
  background-color: #232629;
  border: 1px solid unset;
  border-radius: unset;
  padding: unset;
  padding-left: unset;
  height: unset;
  border-width: unset;
  border-top-left-radius: unset;
  border-top-right-radius: unset;
}

/*  ------------------------------------------------------------------------  */
/*  QToolTip  */

QToolTip {
  padding: 4px;
  border: 1px solid #31363b;
  border-radius: 4px;
  color: #f1f1f1;
  background-color: #AF770F;
}

/*  ------------------------------------------------------------------------  */
/*  QDialog  */



QDialog QToolButton:disabled {
  background-color: #232629;
  color: #f1f1f1
}

/*  ------------------------------------------------------------------------  */
/*  Grips  */


QMainWindow::separator:vertical,
QSplitter::handle:horizontal {
  image: url(icon:/primary/splitter-horizontal.svg);
}

QMainWindow::separator:horizontal,
QSplitter::handle:vertical {
  image: url(icon:/primary/splitter-vertical.svg);
}

QSizeGrip {
  image: url(icon:/primary/sizegrip.svg);
  background-color: transparent;
}

QMenuBar QToolButton:hover,
QMenuBar QToolButton:pressed,
QMenuBar QToolButton {
  border-width: 0;
  border-left: 10px;
  border-image: url(icon:/primary/rightarrow2.svg);
  background-color: transparent;
}

/*  ------------------------------------------------------------------------  */
/*  Focus  */

QDateTimeEdit:focus,
QSpinBox:focus,
QDoubleSpinBox:focus,
QLineEdit:focus,
QComboBox:focus {
  color: #ff9500;
  border: 2px solid #ff9500;
  border-width: 0 0 2px 0;

}

QDateTimeEdit::up-button:focus,
QDoubleSpinBox::up-button:focus,
QSpinBox::up-button:focus {
  image: url(icon:/primary/uparrow.svg);
}

QDateTimeEdit::down-button:focus,
QDoubleSpinBox::down-button:focus,
QSpinBox::down-button:focus {
  image: url(icon:/primary/downarrow.svg);
}

QMenu::indicator:focus,
QListView::indicator:focus,
QTableWidget::indicator:focus,
QRadioButton::indicator:focus {
  background-color: rgba(255, 149, 0, 0.2);
  border-radius: 6px;
 }

QCheckBox::indicator:focus {
  background-color: rgba(255, 149, 0, 0.2);
 }